"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _path = _interopRequireDefault(require("path"));

var _Disk = _interopRequireDefault(require("@polkadot/db/Disk"));

var _Memory = _interopRequireDefault(require("@polkadot/db/Memory"));

var _trieDb = _interopRequireDefault(require("@polkadot/trie-db"));

var _util = require("@polkadot/util");

var _block = _interopRequireDefault(require("./block"));

var _state = _interopRequireDefault(require("./state"));

// Copyright 2017-2019 @polkadot/client-chains authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const SPINNER = ['|', '/', '-', '\\'];
const PREPEND = '                                     ';

class Dbs {
  constructor(_ref, chain) {
    let db = _ref.db;
    this.blocks = void 0;
    this.state = void 0;
    this.basePath = void 0;
    this.config = void 0;
    this.config = db;
    this.basePath = db.type === 'disk' ? _path.default.join(db.path, 'chains', chain.id, (0, _util.u8aToHex)(chain.genesisRoot)) : ''; // NOTE blocks compress very well

    this.blocks = (0, _block.default)(this.createBackingDb('block.db', true)); // NOTE state RLP does not compress well here

    this.state = (0, _state.default)(new _trieDb.default(this.createBackingDb('state.db', false)));
    this.blocks.db.open();
    this.state.db.open();
  }

  createBackingDb(name, isCompressed) {
    return this.config.type === 'disk' ? new _Disk.default(this.basePath, name, {
      isCompressed
    }) : new _Memory.default();
  }

  close() {
    this.blocks.db.close();
    this.state.db.close();
  }

  snapshot() {
    if (!this.config.snapshot) {
      return;
    }

    const newDb = new _trieDb.default(this.createBackingDb('state.db.snapshot', false));
    newDb.open();
    this.state.db.snapshot(newDb, this.createProgress());
    this.state.db.close();
    this.state.db.rename(this.basePath, `state.db.backup-${Date.now()}`);
    newDb.close();
    newDb.rename(this.basePath, 'state.db');
    newDb.open();
    this.state.db = newDb;
  }

  createProgress() {
    let lastUpdate = 0;
    let spin = 0;
    return progress => {
      const now = Date.now();

      if (now - lastUpdate > 200) {
        const percent = `      ${progress.percent.toFixed(2)}`.slice(-6);
        const keys = progress.keys > 9999 ? `${(progress.keys / 1000).toFixed(2)}k` : progress.keys;
        process.stdout.write(`${PREPEND}${SPINNER[spin % SPINNER.length]} ${percent}%, ${keys} keys\r`);
        lastUpdate = now;
        spin++;
      }
    };
  }

}

exports.default = Dbs;